// This file is part of VKF, a CPython library
// for Machine Learning based on Lattice Theory.
//
// This Source Code Form is subject to the terms of the X11 (MIT) License.
// If a copy of the X11 License was not distributed with this file, 
// You can obtain one at https://opensource.org/licenses/MIT/.
//
// X11 (MIT) License
// 
// Copyright (c) 2020 Dmitry V. Vinogradov
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

#ifndef VKF_ENCODER_HPP
#define VKF_ENCODER_HPP

#define DEFAULT_HOST "localhost"
#define DEFAULT_PASS "toor"
#define DEFAULT_USER "root"

#include <mariadb++/connection.hpp>
#include <mariadb++/result_set.hpp>

#include "Maps.hpp"

namespace vkf{
class Encoder : public std::vector<Maps> {
    private:
        std::size_t findIndex(std::string &part_name) {
            std::size_t index = this->size();
            for (std::size_t ndx = 0; ndx < index; ndx++) {
                if (this->at(ndx).getAttributeName() == part_name) {
                    index = ndx;
                    break;
                }
            }
            return index;
        }
    public:
        /** default constructor */
        Encoder()=default;
        Encoder(const std::string &encoder_table_name,
                const std::string &database,
                const std::string &db_host = DEFAULT_HOST,
                const std::string &db_user = DEFAULT_USER,
                const std::string &db_pswd = DEFAULT_PASS
                ) {
            // set up the account
            mariadb::account_ref acc = mariadb::account::create(db_host, db_user, db_pswd);
            // create connection
            mariadb::connection_ref con = mariadb::connection::create(acc);
            // open database
            con->execute("USE " + database);
            // obtain content of table encoder
            mariadb::result_set_ref result = con->query("SELECT * FROM " + encoder_table_name);
//            std::cout << result->row_count() << std::endl;
            con->~connection();
            acc->~account();
            // reading from a result set
            std::string part_name; /**< This variable contains the name of an attribute in current row */
            std::string attribute_value; /**< This variable saves the value in current row */
            char value_letter; /**< This variable saves the letter in current row */
            std::size_t num_bits; /**< This variable saves the length of the bitset in current row */
            unsigned long bitset_ulong; /**< This variable saves the bitset in current row */
            std::size_t index = 0;
            while (result->next()) {
                part_name = result->get_string("attribute_name");
                // find index of Maps with name part_name
                index = this->findIndex(part_name);
//                std::vector<Maps>::iterator it = std::find_if(this->begin(), this->end(), [&part_name](const Maps &arg) {return arg.getAttributeName() == part_name; });
                attribute_value = result->get_string("attribute_value");
                value_letter = result->get_string("value_letter")[0];
                num_bits = result->get_signed32("bitset_length");
                bitset_ulong = result->get_signed32("value_bitset");
                if(index == this->size()) {
//                    std::cout << "add new maps" << std::endl;
//                    this->push_back(Maps(part_name, attribute_value, value_letter, num_bits, bitset_ulong));
                    this->emplace_back(part_name, attribute_value, value_letter, num_bits, bitset_ulong);
                } else {
//                    std::cout << "add new value to " << this->at(index).getAttributeName() << std::endl;
                    this->at(index).AddValue(attribute_value, value_letter, num_bits, bitset_ulong);
                }
                // if(it == this->end()) {
                //     std::cout << "add new maps" << std::endl;
                //     this->emplace_back(part_name, attribute_value, value_letter, num_bits, bitset_ulong);
                // } else {
                //     std::cout << "add new value to " << it->getAttributeName() << std::endl;
                //     it->AddValue(attribute_value, value_letter, num_bits, bitset_ulong);
                // }
            }
        }
};
} // namespace vkf
#endif // VKF_ENCODER_HPP
