// This file is part of VKF, a CPython library
// for Machine Learning based on Lattice Theory.
//
// This Source Code Form is subject to the terms of the X11 (MIT) License.
// If a copy of the X11 License was not distributed with this file, 
// You can obtain one at https://opensource.org/licenses/MIT/.
//
// X11 (MIT) License
// 
// Copyright (c) 2020 Dmitry V. Vinogradov
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

#ifndef VKF_OBSTACLES_HPP
#define VKF_OBSTACLES_HPP
// Declare a class list of obstacles to be VKF-hypothesis.
// This list contains negative examples and artificial fragments whose subsets are declared as trivial similarities.

#include "Sample.hpp" // includes boostdll.h, general.h, and Encoded.hpp too

namespace vkf {
    typedef std::vector<FragmentPtr> ExamplesList;

    class Obstacles : public ExamplesList {
        public:
            /** default constructor */
            Obstacles() = default; // for Induction(Hypotheses&& rhs) constructor
//            Obstacles() = delete; // error: Induction(Hypotheses&& rhs) constructor uses deleted function
            /** copy-constructor */
            Obstacles(const Obstacles&) = delete;
            /** move-constructor */
            Obstacles(Obstacles&&) = default;
            /** copy-operator= */
            Obstacles& operator=(const Obstacles& rhs) = delete;
            /** move-operator= */
            Obstacles& operator=(Obstacles&& rhs) = default;
            /** Constructor from constant Sample lvalue-object declaration */
            Obstacles(const Sample& rhs, const bool is_negative) {
                for (Sample::const_iterator it = rhs.cbegin(); it != rhs.cend(); ++it) {
                    if ((*it)->GetSign() == is_negative) {
                        push_back(vkf::FragmentPtr(new Fragment((boost::dynamic_bitset<>)*(*it))));
//                        std::cout << (boost::dynamic_bitset<>)*(*it) << std::endl;
                    }
                }
//                std::cout << size() << std::endl;
            };
            /** Default destructor */
            ~Obstacles(){
                while(!empty()){
                    pop_back();
               }
            };
    };
} //namespace vkf
#endif // VKF_OBSTACLES_HPP
